/*
 * LabelButton is part of the RADi GUI Builder package.
 * All rights reserved by muntjak Software (www.muntjak.com).
 */
 
package de.muntjak.customcomponents;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.DefaultButtonModel;
import javax.swing.border.Border;
import javax.swing.plaf.basic.BasicGraphicsUtils;

/**
 * LabelButton is a ToggleButton which allows for multilined
 * text. In contrast to Swing HTML text, a LabelButton knows
 * its preferredSize from the beginning.
 * The selected state is visualized by selecting the button
 * text.
 * You seperate lines using the 'plus' sign (e.g. line1+line2).
 * 
 * @version 1.0
 * @author Hans Bickel
 */
public class LabelButton extends JToggleButton {

	private static final Stroke focusStroke = new BasicStroke(
		1.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL,
		0.0f, new float[] {3.0f, 2.0f}, 0.0f);
	protected String text;
	private String[] lines;

	public LabelButton() {
		this("LabelButton", null, false);
	}
	
	public LabelButton(String text) {
		this(text, null, false);
	}
	
	public LabelButton(String text, boolean selected) {
		this(text, null, selected);
	}
	
	public LabelButton(String text, Icon icon) {
		this(text, icon, false);
	}
	
	public LabelButton(String text, Icon icon, boolean select) {
		super(text, icon, select);
		
		this.text = text;
		
		setMargin(new Insets(0, 0, 0, 0));
		setBorder(null);
		setOpaque(false);
		setFocusPainted(false);
		setHorizontalTextPosition(CENTER);
		setVerticalTextPosition(BOTTOM);
	}

	public Dimension getPreferredSize() {
		Icon icon = getIcon();
		Insets insets = getInsets();
		int w = 0, h = 0;
		
		if(icon != null) {
			w = icon.getIconWidth();
			h = icon.getIconHeight();
		}
		
		if(lines == null || lines.length == 0) {
			return new Dimension(w + insets.left + insets.right,
				h + insets.top + insets.bottom);
		}
		
		int texth = 0;
		int textw = 0;
		FontMetrics fm = getToolkit().getFontMetrics(getFont());
		int lh = fm.getHeight();
		int sw;

		for(int i = 0; i < lines.length; i++) {
			sw = fm.stringWidth(lines[i]);
			
			if(sw + 1 > textw) {
				textw = sw + 1;
			}
			
			texth += lh;
		}
		
		if(getVerticalTextPosition() != CENTER) {	// TOP or BOTTOM					
			if(getHorizontalTextPosition() == CENTER) {
				if(textw > w) w = textw;

				h += texth + getIconTextGap();
			}
			else {
				// LEFT | LEADING | RIGHT | TRAILING
				w += textw + getIconTextGap();
				if(texth > h) h = texth;
			}
		}
		else {
			// CENTER
			if(getHorizontalTextPosition() == CENTER) {
				if(textw > w) w = textw;
			}
			else {
				// LEFT | LEADING | RIGHT | TRAILING
				w += textw + getIconTextGap();
			}
			
			if(texth > h) h = texth;
		}

		return new Dimension(w + insets.left + insets.right,
			h + insets.top + insets.bottom);
	}
	
	public Dimension getMinimumSize() {
		return getPreferredSize();
	}

	public void setText(String text) {
		// needed to update displayedMnemonicIndex
		super.setText(text);
		this.text = text;
		
		StringTokenizer t = new StringTokenizer(text, "+");
		lines = new String[t.countTokens()];
		int index = 0;
		
		while(t.hasMoreTokens()) {
			lines[index++] = t.nextToken();
		}
	}

	public String getText() {
		return text;
	}
	
	public void paintComponent(Graphics g) {
		Dimension size = getSize();
		Dimension pref = getPreferredSize();
		Insets insets = getInsets();

		if(isOpaque()) {
			g.setColor(getBackground());
			g.fillRect(0, 0, size.width, size.height);
		}
		
		g.setFont(getFont());
		FontMetrics fm = g.getFontMetrics();
		
		Icon icon = getIcon();
			
		if(isSelected()) {
			if(getModel().isRollover() && getRolloverSelectedIcon() != null) {
				icon = getRolloverSelectedIcon();
			}
			else if(!isEnabled() && getDisabledSelectedIcon() != null) {
				icon = getDisabledSelectedIcon();
			}
			else if(getSelectedIcon() != null) {
				icon = getSelectedIcon();
			}
		}
		else if(!isEnabled() && getDisabledIcon() != null) {
			icon = getDisabledIcon();
		}
		else if(getModel().isPressed() && getPressedIcon() != null) {
			icon = getPressedIcon();
		}
		else if(getModel().isRollover() && getRolloverIcon() != null) {
			icon = getRolloverIcon();
		}

		// calculate text position
		int iconx = 0;
		int icony = 0;
		int iconw = (icon == null ? 0 : icon.getIconWidth());
		int iconh = (icon == null ? 0 : icon.getIconHeight());
		
		int textx = 0;
		int texty = 0;
		int textw = 0;
		int texth = 0;
		
		if(lines != null && lines.length > 0) {
			int lh = fm.getHeight();
			int sw;

			for(int i = 0; i < lines.length; i++) {
				sw = fm.stringWidth(lines[i]);
				
				if(sw > textw) {
					textw = sw;
				}
				
				texth += lh;
			}
		}
		
		if(getVerticalTextPosition() == TOP) {
			if(getHorizontalTextPosition() == LEFT ||
				getHorizontalTextPosition() == LEADING)
			{
				iconx = textw + getIconTextGap();
			}
			else if(getHorizontalTextPosition() == RIGHT ||
				getHorizontalTextPosition() == TRAILING)
			{
				textx = iconw + getIconTextGap();
			}
			else {
				// CENTER
				icony = texth + getIconTextGap();
				
				if(textw > iconw) {
					iconx = (textw - iconw) / 2;
				}
				else {
					textx = (iconw - textw) / 2;				
				}
			}
		}
		else if(getVerticalTextPosition() == BOTTOM) {
			texty = pref.height - texth - insets.top - insets.bottom;
			
			if(getHorizontalTextPosition() == LEFT ||
				getHorizontalTextPosition() == LEADING)
			{
				iconx = textw + getIconTextGap();
				icony = pref.height - iconh - insets.top - insets.bottom;
			}
			else if(getHorizontalTextPosition() == RIGHT ||
				getHorizontalTextPosition() == TRAILING)
			{
				textx = iconw + getIconTextGap();
				icony = pref.height - iconh - insets.top - insets.bottom;
			}
			else {
				// CENTER
				texty = iconh + getIconTextGap();
				
				if(textw > iconw) {
					iconx = (textw - iconw) / 2;
				}
				else {
					textx = (iconw - textw) / 2;
				}
			}
		}
		else {
			// verticalTextPosition = CENTER
			texty = (pref.height - texth) / 2 - insets.top;
			icony = (pref.height - iconh) / 2 - insets.top;

			if(getHorizontalTextPosition() == LEFT ||
				getHorizontalTextPosition() == LEADING)
			{
				iconx = textw + getIconTextGap();
			}
			else if(getHorizontalTextPosition() == RIGHT ||
				getHorizontalTextPosition() == TRAILING)
			{
				textx = iconw + getIconTextGap();
			}
			else {
				// CENTER
				if(textw > iconw) {
					iconx = (textw - iconw) / 2;
				}
				else {
					textx = (iconw - textw) / 2;
				}
			}
		}
		
		iconx += insets.left;
		textx += insets.left + 1;
		icony += insets.top;
		texty += insets.top;
		int translatex = 0;
		int translatey = 0;
		
		if(size.width > pref.width) {
			if(getHorizontalAlignment() == RIGHT || getHorizontalAlignment() == TRAILING) {
				translatex = size.width - pref.width;
			}
			else if(getHorizontalAlignment() == CENTER) {
				translatex = (size.width - pref.width) / 2;
			}
		}
		
		if(size.height > pref.height) {
			if(getVerticalAlignment() == BOTTOM) {
				translatey = size.height - pref.height;
			}
			else if(getVerticalAlignment() == CENTER) {
				translatey = (size.height - pref.height) / 2;
			}
		}

		g.translate(translatex, translatey);
		
		// paint icon, if any
		if(icon != null) {
			icon.paintIcon(this, g, iconx, icony);
		}

		int dmiStart = 0;
		int dmi = getDisplayedMnemonicIndex();

		// paint text, if any
		if(isSelected()) {
			g.setColor(UIManager.getColor("TextField.selectionBackground"));					
			g.fillRect(textx - 1, texty, textw + 1, texth);
		}
		
		if(lines != null && lines.length > 0) {
			int lh = fm.getHeight();
			int y = texty;
			
			for(int i = 0; i < lines.length; i++) {
				int sw = fm.stringWidth(lines[i]);
				int x = textx;

				if(getHorizontalTextPosition() == CENTER) {
					x = textx + (textw - sw) / 2;
				}
				else if(getHorizontalTextPosition() == RIGHT ||
					getHorizontalTextPosition() == TRAILING)
				{
					x = textx + textw - sw;
				}

				if(isSelected()) {
					g.setColor(UIManager.getColor("TextField.selectionForeground"));
				}
				else {
					g.setColor(getForeground());
				}
				
				
				int index = -1;
				if(dmi >= dmiStart && dmi < dmiStart + lines[i].length()) {
					// this is the line which displays the mnemonic
					index = dmi - dmiStart;
				}
				
				BasicGraphicsUtils.drawStringUnderlineCharAt(
					g, lines[i], index, x, y + fm.getAscent());
				
				y += lh;
				dmiStart += lines[i].length() + 1;	// add plus-sign
			}
		}
		
		if(hasFocus() && isFocusPainted()) {
			Graphics2D g2d = (Graphics2D)g;
			Stroke temp = g2d.getStroke();
			g2d.setStroke(focusStroke);
			g2d.setColor(Color.BLACK);
			
			g2d.drawRect(textx - 1, texty, textw, texth - 1);
			g2d.setStroke(temp);
		}
		
		g.translate(-translatex, -translatey);
	}
	
	public static String p(Insets i) {
		return i.top + ", " + i.left + ", " + i.bottom + ", " + i.right;
	}
	
	public static String p(Dimension d) {
		return d.width + ", " + d.height;
	}
}
