/*
 * LinkLabel is part of the RADi GUI Builder package.
 * All rights reserved by muntjak Software (www.muntjak.com).
 */
package de.muntjak.customcomponents;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

/**
 * LinkLabel is a label whose text is always underlined and
 * changes color with rollover.
 * The hand cursor is set by default.
 * If you want to execute an action when the label is clicked,
 * add an ActionListener.
 * 
 * @version 1.0
 * @author Hans Bickel
 */
public class LinkLabel extends JLabel {
	
	// public property
	protected Color rolloverColor = Color.RED;
	protected Color pressedColor = rolloverColor;
	
	// private property
	private Color fgColor = Color.BLUE;
	
	// stores our ActionListeners
	protected EventListenerList listenerList = new EventListenerList();
	
	// to keep track of the mouse state
	private boolean inside = false;
	private boolean pressed = false;
	
	public LinkLabel() {
		super("LinkLabel", null, LEADING);
		
		init();
    }
    
    public LinkLabel(String text) {
		super(text, null, LEADING);
		
		init();
    }
    
    /**
     * Adds an <code>ActionListener</code> to this label.
     * @param l the <code>ActionListener</code> to be added
     */
    public void addActionListener(ActionListener l) {
        listenerList.add(ActionListener.class, l);
    }
    
    /**
     * Removes an <code>ActionListener</code> from the button.
     * If the listener is the currently set <code>Action</code>
     * for the button, then the <code>Action</code>
     * is set to <code>null</code>.
     *
     * @param l the listener to be removed
     */
    public void removeActionListener(ActionListener l) {
		listenerList.remove(ActionListener.class, l);
    }
    
    /**
     * Notifies all listeners that have registered interest for
     * notification on this event type.  The event instance 
     * is lazily created using the <code>event</code> 
     * parameter.
     *
     * @param event  the <code>ActionEvent</code> object
     * @see EventListenerList
     */
    protected void fireActionPerformed(InputEvent event) {
        // Guaranteed to return a non-null array
        Object[] listeners = listenerList.getListenerList();
        ActionEvent e = null;
        
        // Process the listeners last to first, notifying
        // those that are interested in this event
        for(int i = listeners.length - 2; i >= 0; i -= 2) {
            if(listeners[i] == ActionListener.class) {
                // Lazily create the event:
                if (e == null) {
                      e = new ActionEvent(LinkLabel.this,
                          ActionEvent.ACTION_PERFORMED,
                          "",
                          event.getWhen(),
                          event.getModifiers());
                }
                ((ActionListener)listeners[i + 1]).actionPerformed(e);
            }          
        }
    }
    
    private void doEnter() {
    	if(pressed) {
    		super.setForeground(pressedColor);
    	}
    	else {
    		super.setForeground(rolloverColor);
    	}
    	
    	inside = true;
    }
    
    private void doExit() {
    	super.setForeground(fgColor);
    	inside = false;
    }
    
    private void doPressed() {
    	super.setForeground(pressedColor);
    	pressed = true;
    }
    
    private void doReleased(MouseEvent e) {
    	pressed = false;
    	
    	if(inside) {
    		fireActionPerformed(e);
    	}
    }
    
    private void init() {
    	super.setForeground(fgColor);    	
    	super.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
        
        addMouseListener(new MouseAdapter() {
        	public void mouseEntered(MouseEvent e) {
        		doEnter();
        	}
        	
        	public void mouseExited(MouseEvent e) {
        		doExit();
        	}
        	
        	public void mousePressed(MouseEvent e) {
        		if(e.isPopupTrigger()) return;
        		
        		doPressed();
        	}
        	
        	public void mouseReleased(MouseEvent e) {
        		if(e.isPopupTrigger()) return;
        		
        		doReleased(e);
        	}
        });
    }
    
    public void setText(String text) {
    	super.setText(convertToHTML(text));
    }
    
    /*
     * Simple-minded text-to-HTML converter
     */
    private String convertToHTML(String s) {
    	if(s.toLowerCase().startsWith("<html>")) return s;
    	
    	s = s.replaceAll("<", "&lt;");
    	s = s.replaceAll(">", "&gt;");
    	s = s.replaceAll("\"", "&quot;");
    	
    	return "<html><u>" + s;
    }

    public void setForeground(Color fg) {
    	fgColor = fg;
    	
    	super.setForeground(fgColor);
    }

// Getters and Setters
	public Color getRolloverColor() {
		return rolloverColor;
	}

	public void setRolloverColor(Color color) {
		rolloverColor = color;
	}
	
	public Color getPressedColor() {
		return pressedColor;
	}

	public void setPressedColor(Color color) {
		pressedColor = color;
	}
}
